@@ -325,8 +325,10 @@ namespace agg
 
         if(dx >= dx_limit || dx <= -dx_limit)
         {
-            int cx = (x1 + x2) >> 1;
-            int cy = (y1 + y2) >> 1;
+            // These are overflow safe versions of (x1 + x2) >> 1; divide each by 2
+            // first, then add 1 if both were odd.
+            int cx = (x1 >> 1) + (x2 >> 1) + ((x1 & 1) & (x2 & 1));
+            int cy = (y1 >> 1) + (y2 >> 1) + ((y1 & 1) & (y2 & 1));
             line(x1, y1, cx, cy);
             line(cx, cy, x2, y2);
             return;
