@@ -54,7 +54,7 @@ def _prism_blue(x): return -1.1 * np.sin((x * 20.9) * np.pi)
 _prism_data = {'red': _prism_red, 'green': _prism_green, 'blue': _prism_blue}
 
 def _ch_helper(gamma, s, r, h, p0, p1, x):
-    """Helper function for generating picklable cubehelix color maps."""
+    """Helper function for generating picklable cubehelix colormaps."""
     # Apply gamma factor to emphasise low or high intensity values
     xg = x ** gamma
     # Calculate amplitude and angle of deviation from the black to white
@@ -66,7 +66,7 @@ def _ch_helper(gamma, s, r, h, p0, p1, x):
 def cubehelix(gamma=1.0, s=0.5, r=-1.5, h=1.0):
     """
     Return custom data dictionary of (r, g, b) conversion functions, which can
-    be used with :func:`register_cmap`, for the cubehelix color scheme.
+    be used with `.ColormapRegistry.register`, for the cubehelix color scheme.
 
     Unlike most other color schemes cubehelix was designed by D.A. Green to
     be monotonically increasing in terms of perceived brightness.
@@ -76,7 +76,7 @@ def cubehelix(gamma=1.0, s=0.5, r=-1.5, h=1.0):
     can be visualised as a squashed helix around the diagonal in the
     (r, g, b) color cube.
 
-    For a unit color cube (i.e. 3-D coordinates for (r, g, b) each in the
+    For a unit color cube (i.e. 3D coordinates for (r, g, b) each in the
     range 0 to 1) the color scheme starts at (r, g, b) = (0, 0, 0), i.e. black,
     and finishes at (r, g, b) = (1, 1, 1), i.e. white. For some fraction *x*,
     between 0 and 1, the color is the corresponding grey value at that
@@ -155,7 +155,7 @@ def _g34(x): return 2 * x
 def _g35(x): return 2 * x - 0.5
 def _g36(x): return 2 * x - 1
 
-gfunc = {i: globals()["_g{}".format(i)] for i in range(37)}
+gfunc = {i: globals()[f"_g{i}"] for i in range(37)}
 
 _gnuplot_data = {
         'red': gfunc[7],
@@ -459,44 +459,50 @@ _winter_data = {'red':   ((0., 0., 0.), (1.0, 0.0, 0.0)),
                 'blue':  ((0., 1., 1.), (1.0, 0.5, 0.5))}
 
 _nipy_spectral_data = {
-      'red': [(0.0, 0.0, 0.0), (0.05, 0.4667, 0.4667),
-              (0.10, 0.5333, 0.5333), (0.15, 0.0, 0.0),
-              (0.20, 0.0, 0.0), (0.25, 0.0, 0.0),
-              (0.30, 0.0, 0.0), (0.35, 0.0, 0.0),
-              (0.40, 0.0, 0.0), (0.45, 0.0, 0.0),
-              (0.50, 0.0, 0.0), (0.55, 0.0, 0.0),
-              (0.60, 0.0, 0.0), (0.65, 0.7333, 0.7333),
-              (0.70, 0.9333, 0.9333), (0.75, 1.0, 1.0),
-              (0.80, 1.0, 1.0), (0.85, 1.0, 1.0),
-              (0.90, 0.8667, 0.8667), (0.95, 0.80, 0.80),
-              (1.0, 0.80, 0.80)],
-    'green': [(0.0, 0.0, 0.0), (0.05, 0.0, 0.0),
-              (0.10, 0.0, 0.0), (0.15, 0.0, 0.0),
-              (0.20, 0.0, 0.0), (0.25, 0.4667, 0.4667),
-              (0.30, 0.6000, 0.6000), (0.35, 0.6667, 0.6667),
-              (0.40, 0.6667, 0.6667), (0.45, 0.6000, 0.6000),
-              (0.50, 0.7333, 0.7333), (0.55, 0.8667, 0.8667),
-              (0.60, 1.0, 1.0), (0.65, 1.0, 1.0),
-              (0.70, 0.9333, 0.9333), (0.75, 0.8000, 0.8000),
-              (0.80, 0.6000, 0.6000), (0.85, 0.0, 0.0),
-              (0.90, 0.0, 0.0), (0.95, 0.0, 0.0),
-              (1.0, 0.80, 0.80)],
-     'blue': [(0.0, 0.0, 0.0), (0.05, 0.5333, 0.5333),
-              (0.10, 0.6000, 0.6000), (0.15, 0.6667, 0.6667),
-              (0.20, 0.8667, 0.8667), (0.25, 0.8667, 0.8667),
-              (0.30, 0.8667, 0.8667), (0.35, 0.6667, 0.6667),
-              (0.40, 0.5333, 0.5333), (0.45, 0.0, 0.0),
-              (0.5, 0.0, 0.0), (0.55, 0.0, 0.0),
-              (0.60, 0.0, 0.0), (0.65, 0.0, 0.0),
-              (0.70, 0.0, 0.0), (0.75, 0.0, 0.0),
-              (0.80, 0.0, 0.0), (0.85, 0.0, 0.0),
-              (0.90, 0.0, 0.0), (0.95, 0.0, 0.0),
-              (1.0, 0.80, 0.80)],
+    'red': [
+        (0.0, 0.0, 0.0), (0.05, 0.4667, 0.4667),
+        (0.10, 0.5333, 0.5333), (0.15, 0.0, 0.0),
+        (0.20, 0.0, 0.0), (0.25, 0.0, 0.0),
+        (0.30, 0.0, 0.0), (0.35, 0.0, 0.0),
+        (0.40, 0.0, 0.0), (0.45, 0.0, 0.0),
+        (0.50, 0.0, 0.0), (0.55, 0.0, 0.0),
+        (0.60, 0.0, 0.0), (0.65, 0.7333, 0.7333),
+        (0.70, 0.9333, 0.9333), (0.75, 1.0, 1.0),
+        (0.80, 1.0, 1.0), (0.85, 1.0, 1.0),
+        (0.90, 0.8667, 0.8667), (0.95, 0.80, 0.80),
+        (1.0, 0.80, 0.80),
+    ],
+    'green': [
+        (0.0, 0.0, 0.0), (0.05, 0.0, 0.0),
+        (0.10, 0.0, 0.0), (0.15, 0.0, 0.0),
+        (0.20, 0.0, 0.0), (0.25, 0.4667, 0.4667),
+        (0.30, 0.6000, 0.6000), (0.35, 0.6667, 0.6667),
+        (0.40, 0.6667, 0.6667), (0.45, 0.6000, 0.6000),
+        (0.50, 0.7333, 0.7333), (0.55, 0.8667, 0.8667),
+        (0.60, 1.0, 1.0), (0.65, 1.0, 1.0),
+        (0.70, 0.9333, 0.9333), (0.75, 0.8000, 0.8000),
+        (0.80, 0.6000, 0.6000), (0.85, 0.0, 0.0),
+        (0.90, 0.0, 0.0), (0.95, 0.0, 0.0),
+        (1.0, 0.80, 0.80),
+    ],
+    'blue': [
+        (0.0, 0.0, 0.0), (0.05, 0.5333, 0.5333),
+        (0.10, 0.6000, 0.6000), (0.15, 0.6667, 0.6667),
+        (0.20, 0.8667, 0.8667), (0.25, 0.8667, 0.8667),
+        (0.30, 0.8667, 0.8667), (0.35, 0.6667, 0.6667),
+        (0.40, 0.5333, 0.5333), (0.45, 0.0, 0.0),
+        (0.5, 0.0, 0.0), (0.55, 0.0, 0.0),
+        (0.60, 0.0, 0.0), (0.65, 0.0, 0.0),
+        (0.70, 0.0, 0.0), (0.75, 0.0, 0.0),
+        (0.80, 0.0, 0.0), (0.85, 0.0, 0.0),
+        (0.90, 0.0, 0.0), (0.95, 0.0, 0.0),
+        (1.0, 0.80, 0.80),
+    ],
 }
 
 
 # 34 colormaps based on color specifications and designs
-# developed by Cynthia Brewer (http://colorbrewer.org).
+# developed by Cynthia Brewer (https://colorbrewer2.org/).
 # The ColorBrewer palettes have been included under the terms
 # of an Apache-stype license (for details, see the file
 # LICENSE_COLORBREWER in the license directory of the matplotlib
@@ -959,52 +965,55 @@ _Set3_data = (
 # gist_earth_data and gist_ncar_data were simplified by a script and some
 # manual effort.
 
-_gist_earth_data = \
-{'red': (
-(0.0, 0.0, 0.0000),
-(0.2824, 0.1882, 0.1882),
-(0.4588, 0.2714, 0.2714),
-(0.5490, 0.4719, 0.4719),
-(0.6980, 0.7176, 0.7176),
-(0.7882, 0.7553, 0.7553),
-(1.0000, 0.9922, 0.9922),
-), 'green': (
-(0.0, 0.0, 0.0000),
-(0.0275, 0.0000, 0.0000),
-(0.1098, 0.1893, 0.1893),
-(0.1647, 0.3035, 0.3035),
-(0.2078, 0.3841, 0.3841),
-(0.2824, 0.5020, 0.5020),
-(0.5216, 0.6397, 0.6397),
-(0.6980, 0.7171, 0.7171),
-(0.7882, 0.6392, 0.6392),
-(0.7922, 0.6413, 0.6413),
-(0.8000, 0.6447, 0.6447),
-(0.8078, 0.6481, 0.6481),
-(0.8157, 0.6549, 0.6549),
-(0.8667, 0.6991, 0.6991),
-(0.8745, 0.7103, 0.7103),
-(0.8824, 0.7216, 0.7216),
-(0.8902, 0.7323, 0.7323),
-(0.8980, 0.7430, 0.7430),
-(0.9412, 0.8275, 0.8275),
-(0.9569, 0.8635, 0.8635),
-(0.9647, 0.8816, 0.8816),
-(0.9961, 0.9733, 0.9733),
-(1.0000, 0.9843, 0.9843),
-), 'blue': (
-(0.0, 0.0, 0.0000),
-(0.0039, 0.1684, 0.1684),
-(0.0078, 0.2212, 0.2212),
-(0.0275, 0.4329, 0.4329),
-(0.0314, 0.4549, 0.4549),
-(0.2824, 0.5004, 0.5004),
-(0.4667, 0.2748, 0.2748),
-(0.5451, 0.3205, 0.3205),
-(0.7843, 0.3961, 0.3961),
-(0.8941, 0.6651, 0.6651),
-(1.0000, 0.9843, 0.9843),
-)}
+_gist_earth_data = {
+    'red': (
+        (0.0, 0.0, 0.0000),
+        (0.2824, 0.1882, 0.1882),
+        (0.4588, 0.2714, 0.2714),
+        (0.5490, 0.4719, 0.4719),
+        (0.6980, 0.7176, 0.7176),
+        (0.7882, 0.7553, 0.7553),
+        (1.0000, 0.9922, 0.9922),
+    ),
+    'green': (
+        (0.0, 0.0, 0.0000),
+        (0.0275, 0.0000, 0.0000),
+        (0.1098, 0.1893, 0.1893),
+        (0.1647, 0.3035, 0.3035),
+        (0.2078, 0.3841, 0.3841),
+        (0.2824, 0.5020, 0.5020),
+        (0.5216, 0.6397, 0.6397),
+        (0.6980, 0.7171, 0.7171),
+        (0.7882, 0.6392, 0.6392),
+        (0.7922, 0.6413, 0.6413),
+        (0.8000, 0.6447, 0.6447),
+        (0.8078, 0.6481, 0.6481),
+        (0.8157, 0.6549, 0.6549),
+        (0.8667, 0.6991, 0.6991),
+        (0.8745, 0.7103, 0.7103),
+        (0.8824, 0.7216, 0.7216),
+        (0.8902, 0.7323, 0.7323),
+        (0.8980, 0.7430, 0.7430),
+        (0.9412, 0.8275, 0.8275),
+        (0.9569, 0.8635, 0.8635),
+        (0.9647, 0.8816, 0.8816),
+        (0.9961, 0.9733, 0.9733),
+        (1.0000, 0.9843, 0.9843),
+    ),
+    'blue': (
+        (0.0, 0.0, 0.0000),
+        (0.0039, 0.1684, 0.1684),
+        (0.0078, 0.2212, 0.2212),
+        (0.0275, 0.4329, 0.4329),
+        (0.0314, 0.4549, 0.4549),
+        (0.2824, 0.5004, 0.5004),
+        (0.4667, 0.2748, 0.2748),
+        (0.5451, 0.3205, 0.3205),
+        (0.7843, 0.3961, 0.3961),
+        (0.8941, 0.6651, 0.6651),
+        (1.0000, 0.9843, 0.9843),
+    )
+}
 
 _gist_gray_data = {
         'red': gfunc[3],
@@ -1018,58 +1027,61 @@ def _gist_heat_blue(x): return 4 * x - 3
 _gist_heat_data = {
     'red': _gist_heat_red, 'green': _gist_heat_green, 'blue': _gist_heat_blue}
 
-_gist_ncar_data = \
-{'red': (
-(0.0, 0.0, 0.0000),
-(0.3098, 0.0000, 0.0000),
-(0.3725, 0.3993, 0.3993),
-(0.4235, 0.5003, 0.5003),
-(0.5333, 1.0000, 1.0000),
-(0.7922, 1.0000, 1.0000),
-(0.8471, 0.6218, 0.6218),
-(0.8980, 0.9235, 0.9235),
-(1.0000, 0.9961, 0.9961),
-), 'green': (
-(0.0, 0.0, 0.0000),
-(0.0510, 0.3722, 0.3722),
-(0.1059, 0.0000, 0.0000),
-(0.1569, 0.7202, 0.7202),
-(0.1608, 0.7537, 0.7537),
-(0.1647, 0.7752, 0.7752),
-(0.2157, 1.0000, 1.0000),
-(0.2588, 0.9804, 0.9804),
-(0.2706, 0.9804, 0.9804),
-(0.3176, 1.0000, 1.0000),
-(0.3686, 0.8081, 0.8081),
-(0.4275, 1.0000, 1.0000),
-(0.5216, 1.0000, 1.0000),
-(0.6314, 0.7292, 0.7292),
-(0.6863, 0.2796, 0.2796),
-(0.7451, 0.0000, 0.0000),
-(0.7922, 0.0000, 0.0000),
-(0.8431, 0.1753, 0.1753),
-(0.8980, 0.5000, 0.5000),
-(1.0000, 0.9725, 0.9725),
-), 'blue': (
-(0.0, 0.5020, 0.5020),
-(0.0510, 0.0222, 0.0222),
-(0.1098, 1.0000, 1.0000),
-(0.2039, 1.0000, 1.0000),
-(0.2627, 0.6145, 0.6145),
-(0.3216, 0.0000, 0.0000),
-(0.4157, 0.0000, 0.0000),
-(0.4745, 0.2342, 0.2342),
-(0.5333, 0.0000, 0.0000),
-(0.5804, 0.0000, 0.0000),
-(0.6314, 0.0549, 0.0549),
-(0.6902, 0.0000, 0.0000),
-(0.7373, 0.0000, 0.0000),
-(0.7922, 0.9738, 0.9738),
-(0.8000, 1.0000, 1.0000),
-(0.8431, 1.0000, 1.0000),
-(0.8980, 0.9341, 0.9341),
-(1.0000, 0.9961, 0.9961),
-)}
+_gist_ncar_data = {
+    'red': (
+        (0.0, 0.0, 0.0000),
+        (0.3098, 0.0000, 0.0000),
+        (0.3725, 0.3993, 0.3993),
+        (0.4235, 0.5003, 0.5003),
+        (0.5333, 1.0000, 1.0000),
+        (0.7922, 1.0000, 1.0000),
+        (0.8471, 0.6218, 0.6218),
+        (0.8980, 0.9235, 0.9235),
+        (1.0000, 0.9961, 0.9961),
+    ),
+    'green': (
+        (0.0, 0.0, 0.0000),
+        (0.0510, 0.3722, 0.3722),
+        (0.1059, 0.0000, 0.0000),
+        (0.1569, 0.7202, 0.7202),
+        (0.1608, 0.7537, 0.7537),
+        (0.1647, 0.7752, 0.7752),
+        (0.2157, 1.0000, 1.0000),
+        (0.2588, 0.9804, 0.9804),
+        (0.2706, 0.9804, 0.9804),
+        (0.3176, 1.0000, 1.0000),
+        (0.3686, 0.8081, 0.8081),
+        (0.4275, 1.0000, 1.0000),
+        (0.5216, 1.0000, 1.0000),
+        (0.6314, 0.7292, 0.7292),
+        (0.6863, 0.2796, 0.2796),
+        (0.7451, 0.0000, 0.0000),
+        (0.7922, 0.0000, 0.0000),
+        (0.8431, 0.1753, 0.1753),
+        (0.8980, 0.5000, 0.5000),
+        (1.0000, 0.9725, 0.9725),
+    ),
+    'blue': (
+        (0.0, 0.5020, 0.5020),
+        (0.0510, 0.0222, 0.0222),
+        (0.1098, 1.0000, 1.0000),
+        (0.2039, 1.0000, 1.0000),
+        (0.2627, 0.6145, 0.6145),
+        (0.3216, 0.0000, 0.0000),
+        (0.4157, 0.0000, 0.0000),
+        (0.4745, 0.2342, 0.2342),
+        (0.5333, 0.0000, 0.0000),
+        (0.5804, 0.0000, 0.0000),
+        (0.6314, 0.0549, 0.0549),
+        (0.6902, 0.0000, 0.0000),
+        (0.7373, 0.0000, 0.0000),
+        (0.7922, 0.9738, 0.9738),
+        (0.8000, 1.0000, 1.0000),
+        (0.8431, 1.0000, 1.0000),
+        (0.8980, 0.9341, 0.9341),
+        (1.0000, 0.9961, 0.9961),
+    )
+}
 
 _gist_rainbow_data = (
         (0.000, (1.00, 0.00, 0.16)),
@@ -1096,7 +1108,7 @@ _gist_stern_data = {
 def _gist_yarg(x): return 1 - x
 _gist_yarg_data = {'red': _gist_yarg, 'green': _gist_yarg, 'blue': _gist_yarg}
 
-# This bipolar color map was generated from CoolWarmFloat33.csv of
+# This bipolar colormap was generated from CoolWarmFloat33.csv of
 # "Diverging Color Maps for Scientific Visualization" by Kenneth Moreland.
 # <http://www.kennethmoreland.com/color-maps/>
 _coolwarm_data = {
@@ -1207,7 +1219,7 @@ _coolwarm_data = {
 # Implementation of Carey Rappaport's CMRmap.
 # See `A Color Map for Effective Black-and-White Rendering of Color-Scale
 # Images' by Carey Rappaport
-# http://www.mathworks.com/matlabcentral/fileexchange/2662-cmrmap-m
+# https://www.mathworks.com/matlabcentral/fileexchange/2662-cmrmap-m
 _CMRmap_data = {'red':    ((0.000, 0.00, 0.00),
                            (0.125, 0.15, 0.15),
                            (0.250, 0.30, 0.30),
@@ -1239,7 +1251,7 @@ _CMRmap_data = {'red':    ((0.000, 0.00, 0.00),
 
 # An MIT licensed, colorblind-friendly heatmap from Wistia:
 #   https://github.com/wistia/heatmap-palette
-#   http://wistia.com/blog/heatmaps-for-colorblindness
+#   https://wistia.com/learn/culture/heatmaps-for-colorblindness
 #
 # >>> import matplotlib.colors as c
 # >>> colors = ["#e4ff7a", "#ffe81a", "#ffbd00", "#ffa000", "#fc7f00"]
@@ -1354,6 +1366,20 @@ _tab20c_data = (
 )
 
 
+_petroff10_data = (
+    (0.24705882352941178, 0.5647058823529412,  0.8549019607843137),    # 3f90da
+    (1.0,                 0.6627450980392157,  0.054901960784313725),  # ffa90e
+    (0.7411764705882353,  0.12156862745098039, 0.00392156862745098),   # bd1f01
+    (0.5803921568627451,  0.6431372549019608,  0.6352941176470588),    # 94a4a2
+    (0.5137254901960784,  0.17647058823529413, 0.7137254901960784),    # 832db6
+    (0.6627450980392157,  0.4196078431372549,  0.34901960784313724),   # a96b59
+    (0.9058823529411765,  0.38823529411764707, 0.0),                   # e76300
+    (0.7254901960784313,  0.6745098039215687,  0.4392156862745098),    # b9ac70
+    (0.44313725490196076, 0.4588235294117647,  0.5058823529411764),    # 717581
+    (0.5725490196078431,  0.8549019607843137,  0.8666666666666667),    # 92dadd
+)
+
+
 datad = {
     'Blues': _Blues_data,
     'BrBG': _BrBG_data,
