@@ -10,11 +10,13 @@ import numpy as np
 
 import matplotlib as mpl
 from matplotlib.figure import Figure
+from matplotlib.patches import Circle
 from matplotlib.text import Text
 import matplotlib.pyplot as plt
 from matplotlib.testing.decorators import check_figures_equal, image_comparison
 from matplotlib.testing._markers import needs_usetex
 from matplotlib import font_manager as fm
+from matplotlib.offsetbox import (OffsetImage, AnnotationBbox)
 
 
 def test_visibility():
@@ -61,7 +63,7 @@ def test_text_urls():
         fig.savefig(fd, format='svg')
         buf = fd.getvalue().decode()
 
-    expected = '<a xlink:href="{0}">'.format(test_url)
+    expected = f'<a xlink:href="{test_url}">'
     assert expected in buf
 
 
@@ -122,6 +124,27 @@ def test_rasterized_ordering(fig_test, fig_ref):
     ax_test.plot(x+1, y, "-", c="b", lw=10, rasterized=False, zorder=1.2)
 
 
+@check_figures_equal(tol=5, extensions=['svg', 'pdf'])
+def test_prevent_rasterization(fig_test, fig_ref):
+    loc = [0.05, 0.05]
+
+    ax_ref = fig_ref.subplots()
+
+    ax_ref.plot([loc[0]], [loc[1]], marker="x", c="black", zorder=2)
+
+    b = mpl.offsetbox.TextArea("X")
+    abox = mpl.offsetbox.AnnotationBbox(b, loc, zorder=2.1)
+    ax_ref.add_artist(abox)
+
+    ax_test = fig_test.subplots()
+    ax_test.plot([loc[0]], [loc[1]], marker="x", c="black", zorder=2,
+                 rasterized=True)
+
+    b = mpl.offsetbox.TextArea("X")
+    abox = mpl.offsetbox.AnnotationBbox(b, loc, zorder=2.1)
+    ax_test.add_artist(abox)
+
+
 def test_count_bitmaps():
     def count_tag(fig, tag):
         with BytesIO() as fd:
@@ -277,6 +300,33 @@ def test_gid():
             assert gid in buf
 
 
+def test_clip_path_ids_reuse():
+    fig, circle = Figure(), Circle((0, 0), radius=10)
+    for i in range(5):
+        ax = fig.add_subplot()
+        aimg = ax.imshow([[i]])
+        aimg.set_clip_path(circle)
+
+    inner_circle = Circle((0, 0), radius=1)
+    ax = fig.add_subplot()
+    aimg = ax.imshow([[0]])
+    aimg.set_clip_path(inner_circle)
+
+    with BytesIO() as fd:
+        fig.savefig(fd, format='svg')
+        buf = fd.getvalue()
+
+    tree = xml.etree.ElementTree.fromstring(buf)
+    ns = 'http://www.w3.org/2000/svg'
+
+    clip_path_ids = set()
+    for node in tree.findall(f'.//{{{ns}}}clipPath[@id]'):
+        node_id = node.attrib['id']
+        assert node_id not in clip_path_ids  # assert ID uniqueness
+        clip_path_ids.add(node_id)
+    assert len(clip_path_ids) == 2  # only two clipPaths despite reuse in multiple axes
+
+
 def test_savefig_tight():
     # Check that the draw-disabled renderer correctly disables open/close_group
     # as well.
@@ -293,13 +343,17 @@ def test_url():
     s.set_urls(['https://example.com/foo', 'https://example.com/bar', None])
 
     # Line2D
-    p, = plt.plot([1, 3], [6, 5])
+    p, = plt.plot([2, 3, 4], [4, 5, 6])
     p.set_url('https://example.com/baz')
 
+    # Line2D markers-only
+    p, = plt.plot([3, 4, 5], [4, 5, 6], linestyle='none', marker='x')
+    p.set_url('https://example.com/quux')
+
     b = BytesIO()
     fig.savefig(b, format='svg')
     b = b.getvalue()
-    for v in [b'foo', b'bar', b'baz']:
+    for v in [b'foo', b'bar', b'baz', b'quux']:
         assert b'https://example.com/' + v in b
 
 
@@ -581,10 +635,73 @@ def test_svg_font_string(font_str, include_generic):
     text_count = 0
     for text_element in tree.findall(f".//{{{ns}}}text"):
         text_count += 1
-        font_info = dict(
+        font_style = dict(
             map(lambda x: x.strip(), _.strip().split(":"))
             for _ in dict(text_element.items())["style"].split(";")
-        )["font"]
+        )
 
-        assert font_info == f"{size}px {font_str}"
+        assert font_style["font-size"] == f"{size}px"
+        assert font_style["font-family"] == font_str
     assert text_count == len(ax.texts)
+
+
+def test_annotationbbox_gid():
+    # Test that object gid appears in the AnnotationBbox
+    # in output svg.
+    fig = plt.figure()
+    ax = fig.add_subplot()
+    arr_img = np.ones((32, 32))
+    xy = (0.3, 0.55)
+
+    imagebox = OffsetImage(arr_img, zoom=0.1)
+    imagebox.image.axes = ax
+
+    ab = AnnotationBbox(imagebox, xy,
+                        xybox=(120., -80.),
+                        xycoords='data',
+                        boxcoords="offset points",
+                        pad=0.5,
+                        arrowprops=dict(
+                            arrowstyle="->",
+                            connectionstyle="angle,angleA=0,angleB=90,rad=3")
+                        )
+    ab.set_gid("a test for issue 20044")
+    ax.add_artist(ab)
+
+    with BytesIO() as fd:
+        fig.savefig(fd, format='svg')
+        buf = fd.getvalue().decode('utf-8')
+
+    expected = '<g id="a test for issue 20044">'
+    assert expected in buf
+
+
+def test_svgid():
+    """Test that `svg.id` rcparam appears in output svg if not None."""
+
+    fig, ax = plt.subplots()
+    ax.plot([1, 2, 3], [3, 2, 1])
+    fig.canvas.draw()
+
+    # Default: svg.id = None
+    with BytesIO() as fd:
+        fig.savefig(fd, format='svg')
+        buf = fd.getvalue().decode()
+
+    tree = xml.etree.ElementTree.fromstring(buf)
+
+    assert plt.rcParams['svg.id'] is None
+    assert not tree.findall('.[@id]')
+
+    # String: svg.id = str
+    svg_id = 'a test for issue 28535'
+    plt.rc('svg', id=svg_id)
+
+    with BytesIO() as fd:
+        fig.savefig(fd, format='svg')
+        buf = fd.getvalue().decode()
+
+    tree = xml.etree.ElementTree.fromstring(buf)
+
+    assert plt.rcParams['svg.id'] == svg_id
+    assert tree.findall(f'.[@id="{svg_id}"]')
