@@ -1,6 +1,8 @@
 """
 Tests specific to the patches module.
 """
+import platform
+
 import numpy as np
 from numpy.testing import assert_almost_equal, assert_array_equal
 import pytest
@@ -15,9 +17,6 @@ from matplotlib import (
     collections as mcollections, colors as mcolors, patches as mpatches,
     path as mpath, transforms as mtransforms, rcParams)
 
-import sys
-on_win = (sys.platform == 'win32')
-
 
 def test_Polygon_close():
     #: GitHub issue #1018 identified a bug in the Polygon handling
@@ -104,6 +103,57 @@ def test_corner_center():
     assert_almost_equal(ellipse.get_corners(), corners_rot)
 
 
+def test_ellipse_vertices():
+    # expect 0 for 0 ellipse width, height
+    ellipse = Ellipse(xy=(0, 0), width=0, height=0, angle=0)
+    assert_almost_equal(
+        ellipse.get_vertices(),
+        [(0.0, 0.0), (0.0, 0.0)],
+    )
+    assert_almost_equal(
+        ellipse.get_co_vertices(),
+        [(0.0, 0.0), (0.0, 0.0)],
+    )
+
+    ellipse = Ellipse(xy=(0, 0), width=2, height=1, angle=30)
+    assert_almost_equal(
+        ellipse.get_vertices(),
+        [
+            (
+                ellipse.center[0] + ellipse.width / 4 * np.sqrt(3),
+                ellipse.center[1] + ellipse.width / 4,
+            ),
+            (
+                ellipse.center[0] - ellipse.width / 4 * np.sqrt(3),
+                ellipse.center[1] - ellipse.width / 4,
+            ),
+        ],
+    )
+    assert_almost_equal(
+        ellipse.get_co_vertices(),
+        [
+            (
+                ellipse.center[0] - ellipse.height / 4,
+                ellipse.center[1] + ellipse.height / 4 * np.sqrt(3),
+            ),
+            (
+                ellipse.center[0] + ellipse.height / 4,
+                ellipse.center[1] - ellipse.height / 4 * np.sqrt(3),
+            ),
+        ],
+    )
+    v1, v2 = np.array(ellipse.get_vertices())
+    np.testing.assert_almost_equal((v1 + v2) / 2, ellipse.center)
+    v1, v2 = np.array(ellipse.get_co_vertices())
+    np.testing.assert_almost_equal((v1 + v2) / 2, ellipse.center)
+
+    ellipse = Ellipse(xy=(2.252, -10.859), width=2.265, height=1.98, angle=68.78)
+    v1, v2 = np.array(ellipse.get_vertices())
+    np.testing.assert_almost_equal((v1 + v2) / 2, ellipse.center)
+    v1, v2 = np.array(ellipse.get_co_vertices())
+    np.testing.assert_almost_equal((v1 + v2) / 2, ellipse.center)
+
+
 def test_rotate_rect():
     loc = np.asarray([1.0, 2.0])
     width = 2
@@ -246,8 +296,8 @@ def test_patch_alpha_coloring():
                                edgecolor=(0, 0, 1, 0.75))
     ax.add_patch(patch)
 
-    ax.set_xlim([-1, 2])
-    ax.set_ylim([-1, 2])
+    ax.set_xlim(-1, 2)
+    ax.set_ylim(-1, 2)
 
 
 @image_comparison(['patch_alpha_override'], remove_text=True)
@@ -278,8 +328,8 @@ def test_patch_alpha_override():
                                edgecolor=(0, 0, 1, 0.75))
     ax.add_patch(patch)
 
-    ax.set_xlim([-1, 2])
-    ax.set_ylim([-1, 2])
+    ax.set_xlim(-1, 2)
+    ax.set_ylim(-1, 2)
 
 
 @mpl.style.context('default')
@@ -315,8 +365,8 @@ def test_patch_custom_linestyle():
         facecolor=(1, 0, 0), edgecolor=(0, 0, 1))
     ax.add_patch(patch)
 
-    ax.set_xlim([-1, 2])
-    ax.set_ylim([-1, 2])
+    ax.set_xlim(-1, 2)
+    ax.set_ylim(-1, 2)
 
 
 def test_patch_linestyle_accents():
@@ -387,8 +437,8 @@ def test_wedge_movement():
         assert getattr(w, attr) == new_v
 
 
-# png needs tol>=0.06, pdf tol>=1.617
-@image_comparison(['wedge_range'], remove_text=True, tol=1.65 if on_win else 0)
+@image_comparison(['wedge_range'], remove_text=True,
+                  tol=0 if platform.machine() == 'x86_64' else 0.009)
 def test_wedge_range():
     ax = plt.axes()
 
@@ -413,8 +463,8 @@ def test_wedge_range():
 
         ax.add_artist(wedge)
 
-    ax.set_xlim([-2, 8])
-    ax.set_ylim([-2, 9])
+    ax.set_xlim(-2, 8)
+    ax.set_ylim(-2, 9)
 
 
 def test_patch_str():
@@ -488,14 +538,14 @@ def test_multi_color_hatch():
     rects = ax.bar(range(5), range(1, 6))
     for i, rect in enumerate(rects):
         rect.set_facecolor('none')
-        rect.set_edgecolor('C{}'.format(i))
+        rect.set_edgecolor(f'C{i}')
         rect.set_hatch('/')
 
     ax.autoscale_view()
     ax.autoscale(False)
 
     for i in range(5):
-        with mpl.style.context({'hatch.color': 'C{}'.format(i)}):
+        with mpl.style.context({'hatch.color': f'C{i}'}):
             r = Rectangle((i - .8 / 2, 5), .8, 1, hatch='//', fc='none')
         ax.add_patch(r)
 
@@ -513,7 +563,8 @@ def test_units_rectangle():
     ax.set_ylim([5*U.km, 9*U.km])
 
 
-@image_comparison(['connection_patch.png'], style='mpl20', remove_text=True)
+@image_comparison(['connection_patch.png'], style='mpl20', remove_text=True,
+                  tol=0 if platform.machine() == 'x86_64' else 0.024)
 def test_connection_patch():
     fig, (ax1, ax2) = plt.subplots(1, 2)
 
@@ -555,6 +606,28 @@ def test_connection_patch_fig(fig_test, fig_ref):
     fig_ref.add_artist(con)
 
 
+@check_figures_equal(extensions=["png"])
+def test_connection_patch_pixel_points(fig_test, fig_ref):
+    xyA_pts = (.3, .2)
+    xyB_pts = (-30, -20)
+
+    ax1, ax2 = fig_test.subplots(1, 2)
+    con = mpatches.ConnectionPatch(xyA=xyA_pts, coordsA="axes points", axesA=ax1,
+                                   xyB=xyB_pts, coordsB="figure points",
+                                   arrowstyle="->", shrinkB=5)
+    fig_test.add_artist(con)
+
+    plt.rcParams["savefig.dpi"] = plt.rcParams["figure.dpi"]
+
+    ax1, ax2 = fig_ref.subplots(1, 2)
+    xyA_pix = (xyA_pts[0]*(fig_ref.dpi/72), xyA_pts[1]*(fig_ref.dpi/72))
+    xyB_pix = (xyB_pts[0]*(fig_ref.dpi/72), xyB_pts[1]*(fig_ref.dpi/72))
+    con = mpatches.ConnectionPatch(xyA=xyA_pix, coordsA="axes pixels", axesA=ax1,
+                                   xyB=xyB_pix, coordsB="figure pixels",
+                                   arrowstyle="->", shrinkB=5)
+    fig_ref.add_artist(con)
+
+
 def test_datetime_rectangle():
     # Check that creating a rectangle with timedeltas doesn't fail
     from datetime import datetime, timedelta
@@ -880,3 +953,49 @@ def test_modifying_arc(fig_test, fig_ref):
     fig_test.subplots().add_patch(arc2)
     arc2.set_width(.5)
     arc2.set_angle(20)
+
+
+def test_arrow_set_data():
+    fig, ax = plt.subplots()
+    arrow = mpl.patches.Arrow(2, 0, 0, 10)
+    expected1 = np.array(
+       [[1.9,  0.],
+        [2.1, -0.],
+        [2.1, 8.],
+        [2.3, 8.],
+        [2., 10.],
+        [1.7, 8.],
+        [1.9, 8.],
+        [1.9, 0.]]
+    )
+    assert np.allclose(expected1, np.round(arrow.get_verts(), 2))
+
+    expected2 = np.array(
+        [[0.39, 0.04],
+         [0.61, -0.04],
+         [3.01, 6.36],
+         [3.24, 6.27],
+         [3.5, 8.],
+         [2.56, 6.53],
+         [2.79, 6.44],
+         [0.39, 0.04]]
+    )
+    arrow.set_data(x=.5, dx=3, dy=8, width=1.2)
+    assert np.allclose(expected2, np.round(arrow.get_verts(), 2))
+
+
+@check_figures_equal(extensions=["png", "pdf", "svg", "eps"])
+def test_set_and_get_hatch_linewidth(fig_test, fig_ref):
+    ax_test = fig_test.add_subplot()
+    ax_ref = fig_ref.add_subplot()
+
+    lw = 2.0
+
+    with plt.rc_context({"hatch.linewidth": lw}):
+        ax_ref.add_patch(mpatches.Rectangle((0, 0), 1, 1, hatch="x"))
+
+    ax_test.add_patch(mpatches.Rectangle((0, 0), 1, 1, hatch="x"))
+    ax_test.patches[0].set_hatch_linewidth(lw)
+
+    assert ax_ref.patches[0].get_hatch_linewidth() == lw
+    assert ax_test.patches[0].get_hatch_linewidth() == lw
