@@ -1,4 +1,4 @@
-import copy
+import platform
 import re
 
 import numpy as np
@@ -19,6 +19,8 @@ def test_empty_closed_path():
     path = Path(np.zeros((0, 2)), closed=True)
     assert path.vertices.shape == (0, 2)
     assert path.codes is None
+    assert_array_equal(path.get_extents().extents,
+                       transforms.Bbox.null().extents)
 
 
 def test_readonly_path():
@@ -52,15 +54,32 @@ def test_path_exceptions():
 
 def test_point_in_path():
     # Test #1787
-    verts2 = [(0, 0), (0, 1), (1, 1), (1, 0), (0, 0)]
-
-    path = Path(verts2, closed=True)
+    path = Path._create_closed([(0, 0), (0, 1), (1, 1), (1, 0)])
     points = [(0.5, 0.5), (1.5, 0.5)]
     ret = path.contains_points(points)
     assert ret.dtype == 'bool'
     np.testing.assert_equal(ret, [True, False])
 
 
+@pytest.mark.parametrize(
+    "other_path, inside, inverted_inside",
+    [(Path([(0.25, 0.25), (0.25, 0.75), (0.75, 0.75), (0.75, 0.25), (0.25, 0.25)],
+           closed=True), True, False),
+     (Path([(-0.25, -0.25), (-0.25, 1.75), (1.75, 1.75), (1.75, -0.25), (-0.25, -0.25)],
+           closed=True), False, True),
+     (Path([(-0.25, -0.25), (-0.25, 1.75), (0.5, 0.5),
+            (1.75, 1.75), (1.75, -0.25), (-0.25, -0.25)],
+           closed=True), False, False),
+     (Path([(0.25, 0.25), (0.25, 1.25), (1.25, 1.25), (1.25, 0.25), (0.25, 0.25)],
+           closed=True), False, False),
+     (Path([(0, 0), (0, 1), (1, 1), (1, 0), (0, 0)], closed=True), False, False),
+     (Path([(2, 2), (2, 3), (3, 3), (3, 2), (2, 2)], closed=True), False, False)])
+def test_contains_path(other_path, inside, inverted_inside):
+    path = Path([(0, 0), (0, 1), (1, 1), (1, 0), (0, 0)], closed=True)
+    assert path.contains_path(other_path) is inside
+    assert other_path.contains_path(path) is inverted_inside
+
+
 def test_contains_points_negative_radius():
     path = Path.unit_circle()
 
@@ -100,6 +119,16 @@ def test_exact_extents(path, extents):
     assert np.all(path.get_extents().extents == extents)
 
 
+@pytest.mark.parametrize('ignored_code', [Path.CLOSEPOLY, Path.STOP])
+def test_extents_with_ignored_codes(ignored_code):
+    # Check that STOP and CLOSEPOLY points are ignored when calculating extents
+    # of a path with only straight lines
+    path = Path([[0, 0],
+                 [1, 1],
+                 [2, 2]], [Path.MOVETO, Path.MOVETO, ignored_code])
+    assert np.all(path.get_extents().extents == (0., 0., 1., 1.))
+
+
 def test_point_in_path_nan():
     box = np.array([[0, 0], [1, 0], [1, 1], [0, 1], [0, 0]])
     p = Path(box)
@@ -114,15 +143,15 @@ def test_nonlinear_containment():
     ax.set(xscale="log", ylim=(0, 1))
     polygon = ax.axvspan(1, 10)
     assert polygon.get_path().contains_point(
-        ax.transData.transform((5, .5)), ax.transData)
+        ax.transData.transform((5, .5)), polygon.get_transform())
     assert not polygon.get_path().contains_point(
-        ax.transData.transform((.5, .5)), ax.transData)
+        ax.transData.transform((.5, .5)), polygon.get_transform())
     assert not polygon.get_path().contains_point(
-        ax.transData.transform((50, .5)), ax.transData)
+        ax.transData.transform((50, .5)), polygon.get_transform())
 
 
-@image_comparison(['arrow_contains_point.png'],
-                  remove_text=True, style='mpl20')
+@image_comparison(['arrow_contains_point.png'], remove_text=True, style='mpl20',
+                  tol=0 if platform.machine() == 'x86_64' else 0.027)
 def test_arrow_contains_point():
     # fix bug (#8384)
     fig, ax = plt.subplots()
@@ -194,8 +223,14 @@ def test_log_transform_with_zero():
 def test_make_compound_path_empty():
     # We should be able to make a compound path with no arguments.
     # This makes it easier to write generic path based code.
-    r = Path.make_compound_path()
-    assert r.vertices.shape == (0, 2)
+    empty = Path.make_compound_path()
+    assert empty.vertices.shape == (0, 2)
+    r2 = Path.make_compound_path(empty, empty)
+    assert r2.vertices.shape == (0, 2)
+    assert r2.codes.shape == (0,)
+    r3 = Path.make_compound_path(Path([(0, 0)]), empty)
+    assert r3.vertices.shape == (1, 2)
+    assert r3.codes.shape == (1,)
 
 
 def test_make_compound_path_stops():
@@ -247,7 +282,8 @@ def test_marker_paths_pdf():
 
 
 @image_comparison(['nan_path'], style='default', remove_text=True,
-                  extensions=['pdf', 'svg', 'eps', 'png'])
+                  extensions=['pdf', 'svg', 'eps', 'png'],
+                  tol=0 if platform.machine() == 'x86_64' else 0.009)
 def test_nan_isolated_points():
 
     y0 = [0, np.nan, 2, np.nan, 4, 5, 6]
@@ -321,8 +357,28 @@ def test_path_deepcopy():
     codes = [Path.MOVETO, Path.LINETO]
     path1 = Path(verts)
     path2 = Path(verts, codes)
-    copy.deepcopy(path1)
-    copy.deepcopy(path2)
+    path1_copy = path1.deepcopy()
+    path2_copy = path2.deepcopy()
+    assert path1 is not path1_copy
+    assert path1.vertices is not path1_copy.vertices
+    assert path2 is not path2_copy
+    assert path2.vertices is not path2_copy.vertices
+    assert path2.codes is not path2_copy.codes
+
+
+def test_path_shallowcopy():
+    # Should not raise any error
+    verts = [[0, 0], [1, 1]]
+    codes = [Path.MOVETO, Path.LINETO]
+    path1 = Path(verts)
+    path2 = Path(verts, codes)
+    path1_copy = path1.copy()
+    path2_copy = path2.copy()
+    assert path1 is not path1_copy
+    assert path1.vertices is path1_copy.vertices
+    assert path2 is not path2_copy
+    assert path2.vertices is path2_copy.vertices
+    assert path2.codes is path2_copy.codes
 
 
 @pytest.mark.parametrize('phi', np.concatenate([
@@ -398,6 +454,16 @@ def test_path_intersect_path(phi):
     b = transform.transform_path(Path([(0, 1), (0, 2), (0, 5)]))
     assert a.intersects_path(b) and b.intersects_path(a)
 
+    # a and b are collinear but do not intersect
+    a = transform.transform_path(Path([(1, -1), (0, -1)]))
+    b = transform.transform_path(Path([(0, 1), (0.9, 1)]))
+    assert not a.intersects_path(b) and not b.intersects_path(a)
+
+    # a and b are collinear but do not intersect
+    a = transform.transform_path(Path([(0., -5.), (1., -5.)]))
+    b = transform.transform_path(Path([(1., 5.), (0., 5.)]))
+    assert not a.intersects_path(b) and not b.intersects_path(a)
+
 
 @pytest.mark.parametrize('offset', range(-720, 361, 45))
 def test_full_arc(offset):
