@@ -25,7 +25,7 @@ def download_or_cache(url, version):
     if cache_dir is not None:  # Try to read from cache.
         try:
             data = (cache_dir / version).read_bytes()
-        except IOError:
+        except OSError:
             pass
         else:
             return BytesIO(data)
@@ -40,7 +40,7 @@ def download_or_cache(url, version):
             cache_dir.mkdir(parents=True, exist_ok=True)
             with open(cache_dir / version, "xb") as fout:
                 fout.write(data)
-        except IOError:
+        except OSError:
             pass
 
     return BytesIO(data)
@@ -63,6 +63,22 @@ def _get_xdg_cache_dir():
 
 if __name__ == "__main__":
     data = {
+        "v3.10.0": "14464227",
+        "v3.9.4": "14436121",
+        "v3.9.3": "14249941",
+        "v3.9.2": "13308876",
+        "v3.9.1": "12652732",
+        "v3.9.0": "11201097",
+        "v3.8.4": "10916799",
+        "v3.8.3": "10661079",
+        "v3.8.2": "10150955",
+        "v3.8.1": "10059757",
+        "v3.8.0": "8347255",
+        "v3.7.3": "8336761",
+        "v3.7.2": "8118151",
+        "v3.7.1": "7697899",
+        "v3.7.0": "7637593",
+        "v3.6.3": "7527665",
         "v3.6.2": "7275322",
         "v3.6.1": "7162185",
         "v3.6.0": "7084615",
@@ -113,11 +129,11 @@ if __name__ == "__main__":
     }
     doc_dir = Path(__file__).parent.parent.absolute() / "doc"
     target_dir = doc_dir / "_static/zenodo_cache"
-    citing = doc_dir / "users/project/citing.rst"
+    citing = doc_dir / "project/citing.rst"
     target_dir.mkdir(exist_ok=True, parents=True)
     header = []
     footer = []
-    with open(citing, "r") as fin:
+    with open(citing) as fin:
         target = header
         for ln in fin:
             if target is not None:
@@ -127,7 +143,7 @@ if __name__ == "__main__":
                 target = None
             if ln.strip() == ".. END OF AUTOGENERATED":
                 target = footer
-                target.append(ln)
+                target.append(ln.rstrip())
 
     with open(citing, "w") as fout:
         fout.write("\n".join(header))
@@ -141,9 +157,9 @@ if __name__ == "__main__":
             fout.write(
                 f"""
 {version}
-   .. image:: ../../_static/zenodo_cache/{doi}.svg
+   .. image:: ../_static/zenodo_cache/{doi}.svg
       :target:  https://doi.org/10.5281/zenodo.{doi}"""
             )
         fout.write("\n\n")
         fout.write("\n".join(footer))
-        fout.write('\n')
+        fout.write("\n")
